@@ -150,6 +150,8 @@ public class IconView : Gtk.Layout {
 	public int FocusCell {
 		set {
 			if (value != real_focus_cell) {
+				value = Math.Max (value, 0);
+				value = Math.Min (value, collection.Count - 1);
 				InvalidateCell (value);
 				InvalidateCell (real_focus_cell);
 				real_focus_cell = value;
@@ -164,9 +166,6 @@ public class IconView : Gtk.Layout {
 	// about.
 	private int click_count;
 
-	// The pixbuf we use when we can't load a thumbnail.
-	static Pixbuf error_pixbuf;
-
 	// Public events.
 	public delegate void DoubleClickedHandler (IconView view, int clicked_item);
 	public event DoubleClickedHandler DoubleClicked;
@@ -1451,8 +1450,6 @@ public class IconView : Gtk.Layout {
 		case Gdk.Key.Return:
 			if (DoubleClicked == null)
 				break;
-			if (FocusCell < 0 || FocusCell > collection.Count - 1)
-				break;
 			DoubleClicked (this, FocusCell);
 			break;
 		default:	
@@ -1460,9 +1457,6 @@ public class IconView : Gtk.Layout {
 			return;		
 		}
 		
-		FocusCell = Math.Max (FocusCell, 0);
-		FocusCell = Math.Min (FocusCell, collection.Count - 1);
-		
 		if (FocusCell == focus_old) {
 			args.RetVal = false;
 			return;
