@@ -7,20 +7,27 @@ public class TagMenu : Menu {
 
 	public delegate void TagSelectedHandler (Tag t);
 	public event TagSelectedHandler TagSelected;
+	
+	private EventHandler new_tag_handler = null;
+	public EventHandler NewTagHandler {
+		get { return new_tag_handler; }
+		set { new_tag_handler = value; }
+	}
 
-	public class TagItem : Gtk.ImageMenuItem {
+	public class TagMenuItem : Gtk.ImageMenuItem {
 		public Tag Value;
 
-		public TagItem (Tag t) : this (t, t.Name) { }
+		public TagMenuItem (Tag t) : this (t, t.Name) { }
 		
-		public TagItem (Tag t, string name) : base (name)
+		public TagMenuItem (Tag t, string name) : base (name.Replace ("_", "__"))
 		{
 			Value = t;
 			if (t.Icon != null)
 				this.Image = new Gtk.Image (t.Icon);
 		}
 
-		public static TagItem IndentedItem (Tag t) {
+		public static TagMenuItem IndentedItem (Tag t)
+		{
 			System.Text.StringBuilder label_builder = new System.Text.StringBuilder ();
 			
 			for (Category parent = t.Category; 
@@ -29,10 +36,10 @@ public class TagMenu : Menu {
 				label_builder.Append ("  ");
 			
 			label_builder.Append (t.Name);
-			return new TagItem (t, label_builder.ToString ());
+			return new TagMenuItem (t, label_builder.ToString ());
 		}
 
-		protected TagItem (IntPtr raw) : base (raw) {}
+		protected TagMenuItem (IntPtr raw) : base (raw) {}
 	}
 
 	public TagMenu (MenuItem item, TagStore store) 
@@ -48,21 +55,46 @@ public class TagMenu : Menu {
 
 	protected TagMenu (IntPtr raw) : base (raw) {}
 
-	public void Populate () {
+	public void Populate ()
+	{
 		Populate (false);
 	}
 
-	public void Populate (bool flat) { 
+	public int GetPosition (Tag t)
+	{
+		// FIXME right now this only works on flat menus
+
+		int i = 0;
+		foreach (Widget w in this.Children) {
+			TagMenuItem item = w as TagMenuItem;
+			if (item != null) {
+				if (t == item.Value)
+					return i;
+			}
+			i++;
+		}
+		
+		return -1;
+	}
+
+	public void Populate (bool flat)
+	{ 
 		if (flat)
 			PopulateFlat (tag_store.RootCategory, this);
 		else
 			Populate (tag_store.RootCategory, this);
+
+		if (NewTagHandler != null) {
+			GtkUtil.MakeMenuSeparator (this);
+			GtkUtil.MakeMenuItem (this, Mono.Posix.Catalog.GetString ("Create New Tag"),
+					"f-spot-new-tag", NewTagHandler, true);
+		}
 	}
 
         public void PopulateFlat (Category cat, Gtk.Menu parent)
 	{
 		foreach (Tag t in cat.Children) {
-			TagItem item = TagItem.IndentedItem (t);
+			TagMenuItem item = TagMenuItem.IndentedItem (t);
 			parent.Append (item);
 			item.ShowAll ();
 
@@ -75,13 +107,14 @@ public class TagMenu : Menu {
 		} 
 	}
 
-	public void Populate (Category cat, Gtk.Menu parent) {
-		foreach (Widget w in parent.Children) {
-			w.Destroy ();
-		}
+	public void Populate (Category cat, Gtk.Menu parent)
+	{
+		Widget [] dead_pool = parent.Children;
+		for (int i = 0; i < dead_pool.Length; i++)
+			dead_pool [i].Destroy ();
 
 		foreach (Tag t in cat.Children) {
-			TagItem item = new TagItem (t);
+			TagMenuItem item = new TagMenuItem (t);
 			parent.Append (item);
 			item.ShowAll ();
 
@@ -94,7 +127,7 @@ public class TagMenu : Menu {
 				submenu.Prepend (sep);
 				sep.ShowAll ();
 
-				TagItem subitem = new TagItem (t);
+				TagMenuItem subitem = new TagMenuItem (t);
 				subitem.Activated += HandleActivate;
 				submenu.Prepend (subitem);
 				subitem.ShowAll ();
@@ -106,19 +139,19 @@ public class TagMenu : Menu {
 		} 
 	}
 	
-	private void HandlePopulate (object obj, EventArgs args) {
+	private void HandlePopulate (object obj, EventArgs args)
+	{
 		this.Populate ();
 	}
 	
 	void HandleActivate (object obj, EventArgs args)
 	{
 		if (TagSelected != null) {
-			TagItem t = obj as TagItem;
+			TagMenuItem t = obj as TagMenuItem;
 			if (t != null)
 				TagSelected (t.Value);
 			else 
-				Console.WriteLine ("Item was not a TagItem");
+				Console.WriteLine ("Item was not a TagMenuItem");
 		}
 	}
-
 }
